local class = require "class"
local m = class()

function m.debugf(...)
    io.stderr:write(string.format(...))
end

local v0 = string.byte("0")
local va = string.byte("A") - 10

local function hexchar(v)
    --io.stderr:write("hexchar v=" .. v)
    if v < 10 then
        v = v + v0
    else
        v = v + va
    end
    --io.stderr:write(" => " .. v .. "(" .. string.char(v) .. ")\n")
    return v
end

function m.tohex(s)
    local hex = ""
    if not s
        then s = ""
    end
    local len = #s
    local chars = {}
    for i = 1, len do
        local c = string.byte(s, i)
        hex = hex .. " "
            .. string.char(hexchar(math.floor(c / 16)))
            .. string.char(hexchar(c % 16))
    end
    return hex
end

-- From http://lua-users.org/wiki/TableSerialization

local function table_print(tt, indent, done)
    done = done or {}
    indent = indent or 0
    if type(tt) == "table" then
        local sb = {}
        for key, value in pairs (tt) do
            table.insert(sb, string.rep (" ", indent)) -- indent it
            if type(value) == "table" and not done[value] then
                done[value] = true
                table.insert(sb, string.format("%s = {\n", tostring(key)))
                table.insert(sb, table_print(value, indent + 2, done))
                table.insert(sb, string.rep(" ", indent)) -- indent it
                table.insert(sb, "}\n");
            elseif "number" == type(key) then
                table.insert(sb, string.format("\"%s\"\n", tostring(value)))
            else
                table.insert(sb, string.format("%s = \"%s\"\n",
                    tostring(key), tostring(value)))
            end
        end
        return table.concat(sb)
    else
        return tt .. "\n"
    end
end

function m.dump(tbl)
    if "nil" == type(tbl) then
        return tostring(nil)
    elseif "table" == type(tbl) then
        return table_print(tbl)
    elseif "string" == type(tbl) then
        return tbl
    else
        return tostring(tbl)
    end
end

setmetatable(m, {
    __call = function(t, fmt, ...)
        io.stderr:write(string.format(fmt, ...))
    end,
})

return m
