require 'strict'
local ffi = require 'ffi'
local C = ffi.C
local config = require 'system_config'
local libte = ffi.load(config.test_engine_library)

if config.test_engine_library == '/usr/lib/libte-debug.so' then
ffi.cdef[[
    typedef int64_t bh_offset_t;

    struct hbms_hdr {
        uint32_t        hdr;            // HBM stream header info
//        uint32_t        ts;             // timestamp of first sample in frame
//      FIXME:  Make the ts below a double sometime
        float           ts;             // timestamp of first sample in frame
    } __attribute__((packed));

    struct frame {
        struct frame    *next;          // Next frame in list
        struct frame    *prev;          // Previous frame in list
        struct frame    *root;          // Pointer to root frame (clones only)
        struct frame    **ff_list;      // Pointer to free frame list
        uint16_t        msg_frame;      // != 0 is a message frame
        int16_t         latch;          // reference count
        uint32_t        element_size;   // Size of elements in buffer
        uint32_t        maxsize;        // Maximum elements in frame
        uint32_t        count;          // Number of elements in frame
        bh_offset_t     offset;         // Trigger offset (by Burst History)
        int64_t         ts;             // Total sample count less this frame
        uint32_t        sequence_num;   // Frame sequence number for debug
        void            *buf;           // Buffer containing elements
        struct hbms_hdr *hdr;           // HBM stream header
    } __attribute__((packed));
]]
else
ffi.cdef[[
    typedef int64_t bh_offset_t;

    struct hbms_hdr {
        uint32_t        hdr;            // HBM stream header info
//        uint32_t        ts;             // timestamp of first sample in frame
//      FIXME:  Make the ts below a double sometime
        float           ts;             // timestamp of first sample in frame
    } __attribute__((packed));

    struct frame {
        struct frame    *next;          // Next frame in list
        struct frame    *prev;          // Previous frame in list
        struct frame    *root;          // Pointer to root frame (clones only)
        struct frame    **ff_list;      // Pointer to free frame list
        uint16_t        msg_frame;      // != 0 is a message frame
        int16_t         latch;          // reference count
        uint32_t        element_size;   // Size of elements in buffer
        uint32_t        maxsize;        // Maximum elements in frame
        uint32_t        count;          // Number of elements in frame
        bh_offset_t     offset;         // Trigger offset (by Burst History)
        int64_t         ts;             // Full precision timestamp
        void            *buf;           // Buffer containing elements
        struct hbms_hdr *hdr;           // HBM stream header
    } __attribute__((packed));
]]
end

ffi.cdef[[
    void *calloc(size_t nmemb, size_t size);
    void free(void *ptr);

    // Prototypes for functions in libte
    struct frame *frameCreate(int bufsize, int element_bytes, void *buf);
    void frameDelete(struct frame *frame);
    struct frame *frameClone(struct frame *root);
    int frameBufsize(double sample_rate, double pipe_frame_rate);
    int frameLength(struct frame *frame);
    int frameAvailable(struct frame *frame);
    char *frameTail(struct frame *frame);
]]


return {
    create = function(bufsize, element_bytes)
        return libte.frameCreate(bufsize, element_bytes, nil)
    end,
    delete = libte.frameDelete,
    clone = libte.frameClone,
    bufsize = libte.frameBufsize,
    length = libte.frameLength,
    available = libte.frameAvailable,
    tail = libte.frameTail,
}
