-- time_channel.lua - Implements the Time computed channel.

local dbg = require 'dbg'
local ffi = require 'ffi'
local rational = require 'rational'
local Frame = require 'test_engine/frame'
local Pipe = require 'test_engine/pipe'

local Frame_create = Frame.create
local Frame_clone = Frame.clone
local Pipe_add_frame = Pipe.add_frame
local Pipe_new_pipe_list = Pipe.new_pipe_list

local m = {}

-- Function called to do object's processing
function m:run()
    if self.pipe_list[0] == nil or not self.sie_writer or not
        self.enabled then return end

    -- Create a single entry frame
    local root = Frame_create(1, ffi.sizeof('float'))
    if root == nil then return end

    root.hdr.ts = 0

    -- Add the single entry to the frame
    root.count = 1
    local d = ffi.cast('float *', root.buf)
    d[0] = self.sie_writer:amount_written()

    -- Add the frame to the pipe(s)
    local p = self.pipe_list[0]
    Pipe_add_frame(p, root)
    while p.next ~= nil do
        Pipe_add_frame(p.next, Frame_clone(root))
        p = p.next
    end
end

-- Setup function
function m:setup(sie_writer)
    self.sie_writer = sie_writer
end

-- Function called to instantiate the object
function m:new(stanza, node_list, sie_index)
    self.stanza = stanza
    self.pipe_list = Pipe_new_pipe_list()
    return self, 0
end

return m
