require 'strict'

local ffi = require 'ffi'
local cjson = require 'cjson.safe'
local config = require 'system_config'
local libte = ffi.load(config.test_engine_library)

ffi.cdef[[
    struct _xcp_ip_set_t *xcp_ip_set_create();
    struct _xcp_ip_measurement_t *xcp_ip_set_add_measurement(
        struct _xcp_ip_set_t *set, const char *spec);
    void xcp_ip_set_finalize(struct _xcp_ip_set_t *set);
]]

local m = {}

function m.create(stanza, pipe_rate)
    return libte.xcp_ip_set_create()
end

function m.finalize(set)
    libte.xcp_ip_set_finalize(set)
end

function m.add_measurement(set, stanza, pipe_rate)

    assert(stanza, "Missing parameter 'stanza'")
    assert(pipe_rate, "Missing parameter 'pipe_rate'")

    -- Calculate the aggregate scale and offset
    local slope = 1.0
    local intercept = 0.0
    local csteps = {
        {
            slope = tonumber(stanza.db_scale or 1.0),
            intercept = tonumber(stanza.db_offset or 0.0),
        },
        {
            slope = stanza.scale or 1.0,
            intercept = stanza.offset or 0.0
        },
    }
    for _, cal in ipairs(csteps) do
        slope = slope * cal.slope
        intercept = cal.slope * intercept + cal.intercept
    end
    stanza.scale = slope
    stanza.offset = intercept
    stanza.pipe_rate = pipe_rate
    return libte.xcp_ip_set_add_measurement(set, cjson.encode(stanza) or '{}')

end

return m
